package cn.estsh.i3plus.ext.mes.apiservice.controller.base.bu3;

import cn.estsh.i3plus.ext.mes.api.base.bu3.ISxCareerCorrelationCfgService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesCareerCorrelationCfg;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @Author: wangjie
 * @CreateDate: 2021/01/19 15:37
 * @Description:
 **/
@RestController
@RequestMapping(CommonConstWords.BASE_URL_MES)
@Api(tags = "事业别相关配置控制层")
public class SxCareerCorrelationCfgController extends MesBaseController {

    @Autowired
    private ISxCareerCorrelationCfgService careerCorrelationCfgService;

    @GetMapping(value = "/sx-ext/career-correlation-cfg/query")
    @ApiOperation(value = "按条件分页查询事业别相关配置信息")
    public ResultBean queryCareerCorrelationCfgByPager(MesCareerCorrelationCfg careerCorrelationCfg, Pager pager) {
        try {
            ValidatorBean.beginValid(careerCorrelationCfg)
                    .notNull(MesExtConstWords.DATA_TYPE, careerCorrelationCfg.getDataType());
            careerCorrelationCfg.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setListPager(careerCorrelationCfgService.queryCareerCorrelationCfgByPager(careerCorrelationCfg, pager));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/sx-ext/career-correlation-cfg/insert")
    @ApiOperation(value = "新增事业别相关配置信息")
    public ResultBean insertCareerCorrelationCfg(MesCareerCorrelationCfg careerCorrelationCfg) {
        try {
            ValidatorBean.beginValid(careerCorrelationCfg)
                    .notNull(MesExtConstWords.DATA_TYPE, careerCorrelationCfg.getDataType())
                    .notNull(MesExtConstWords.DATA_KEY, careerCorrelationCfg.getDataKey())
                    .notNull(MesExtConstWords.CFG_VALUE, careerCorrelationCfg.getCfgValue());

            careerCorrelationCfg.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            careerCorrelationCfg.setCreateUser(AuthUtil.getSessionUser().getUserName());
            careerCorrelationCfgService.insertCareerCorrelationCfg(careerCorrelationCfg);

            return ResultBean.success("新增成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/sx-ext/career-correlation-cfg/update")
    @ApiOperation(value = "更新事业别相关配置信息")
    public ResultBean updateCareerCorrelationCfg(MesCareerCorrelationCfg careerCorrelationCfg) {
        try {
            ValidatorBean.beginValid(careerCorrelationCfg)
                    .notNull(MesExtConstWords.ID, careerCorrelationCfg.getIdStr())
                    .notNull(MesExtConstWords.DATA_TYPE, careerCorrelationCfg.getDataType())
                    .notNull(MesExtConstWords.DATA_KEY, careerCorrelationCfg.getDataKey())
                    .notNull(MesExtConstWords.CFG_VALUE, careerCorrelationCfg.getCfgValue());

            careerCorrelationCfg.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            careerCorrelationCfg.setModifyUser(AuthUtil.getSessionUser().getUserName());
            careerCorrelationCfgService.updateCareerCorrelationCfg(careerCorrelationCfg);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/sx-ext/career-correlation-cfg/status-update")
    @ApiOperation(value = "根据id启用或禁用事业别相关配置信息")
    public ResultBean updateCareerCorrelationCfgValidById(String idStr, Integer status) {
        try {
            if (StringUtils.isEmpty(idStr)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }
            careerCorrelationCfgService.updateCareerCorrelationCfgValidById(idStr, status, AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @DeleteMapping(value = "/sx-ext/career-correlation-cfg/delete")
    @ApiOperation(value = "根据id删除事业别相关配置信息")
    public ResultBean deleteCareerCorrelationCfgById(String idStr) {
        try {
            if (StringUtils.isEmpty(idStr)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }
            careerCorrelationCfgService.deleteCareerCorrelationCfgById(idStr, AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("删除成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
