package cn.estsh.i3plus.ext.mes.apiservice.controller.base.bu3;

import cn.estsh.i3plus.ext.mes.api.base.bu3.ISxPartCategoryTargetCfgService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesPartCategoryTargetCfg;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @Author: wangjie
 * @CreateDate: 2021/01/19 15:37
 * @Description:
 **/
@RestController
@RequestMapping(CommonConstWords.BASE_URL_MES)
@Api(tags = "产品类别目标配置控制层")
public class SxPartCategoryTargetCfgController extends MesBaseController {

    @Autowired
    private ISxPartCategoryTargetCfgService partCategoryTargetCfgService;

    @GetMapping(value = "/sx-ext/part-category-target-cfg/query")
    @ApiOperation(value = "按条件分页查询产品类别目标配置信息")
    public ResultBean queryPartCategoryTargetCfgByPager(MesPartCategoryTargetCfg partCategoryTargetCfg, Pager pager) {
        try {
            partCategoryTargetCfg.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setListPager(partCategoryTargetCfgService.queryPartCategoryTargetCfgByPager(partCategoryTargetCfg, pager));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/sx-ext/part-category-target-cfg/insert")
    @ApiOperation(value = "新增产品类别目标配置信息")
    public ResultBean insertPartCategoryTargetCfg(MesPartCategoryTargetCfg partCategoryTargetCfg) {
        try {
            ValidatorBean.beginValid(partCategoryTargetCfg)
                    .notNull(MesExtConstWords.CATEGORY_CODE, partCategoryTargetCfg.getCategoryCode())
                    .notNull(MesExtConstWords.DATA_KEY, partCategoryTargetCfg.getDataKey())
                    .notNull(MesExtConstWords.CFG_VALUE, partCategoryTargetCfg.getCfgValue())
                    .notNull(MesExtConstWords.CFG_VALUE, partCategoryTargetCfg.getCfgValue2());

            partCategoryTargetCfg.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            partCategoryTargetCfg.setCreateUser(AuthUtil.getSessionUser().getUserName());
            partCategoryTargetCfgService.insertPartCategoryTargetCfg(partCategoryTargetCfg);

            return ResultBean.success("新增成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/sx-ext/part-category-target-cfg/update")
    @ApiOperation(value = "更新产品类别目标配置信息")
    public ResultBean updatePartCategoryTargetCfg(MesPartCategoryTargetCfg partCategoryTargetCfg) {
        try {
            ValidatorBean.beginValid(partCategoryTargetCfg)
                    .notNull(MesExtConstWords.ID, partCategoryTargetCfg.getIdStr())
                    .notNull(MesExtConstWords.CATEGORY_CODE, partCategoryTargetCfg.getCategoryCode())
                    .notNull(MesExtConstWords.DATA_KEY, partCategoryTargetCfg.getDataKey())
                    .notNull(MesExtConstWords.CFG_VALUE, partCategoryTargetCfg.getCfgValue())
                    .notNull(MesExtConstWords.CFG_VALUE, partCategoryTargetCfg.getCfgValue2());

            partCategoryTargetCfg.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            partCategoryTargetCfg.setModifyUser(AuthUtil.getSessionUser().getUserName());
            partCategoryTargetCfgService.updatePartCategoryTargetCfg(partCategoryTargetCfg);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/sx-ext/part-category-target-cfg/status-update")
    @ApiOperation(value = "根据id启用或禁用产品类别目标配置信息")
    public ResultBean updatePartCategoryTargetCfgValidById(String idStr, Integer status) {
        try {
            if (StringUtils.isEmpty(idStr)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }
            partCategoryTargetCfgService.updatePartCategoryTargetCfgValidById(idStr, status, AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @DeleteMapping(value = "/sx-ext/part-category-target-cfg/delete")
    @ApiOperation(value = "根据id删除产品类别目标配置信息")
    public ResultBean deletePartCategoryTargetCfgById(String idStr) {
        try {
            if (StringUtils.isEmpty(idStr)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }
            partCategoryTargetCfgService.deletePartCategoryTargetCfgById(idStr, AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("删除成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
