package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IEquipmentCheckService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesEquipmentCheck;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesEquipmentCheckRepository;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.sqlpack.MesHqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

/**
 * @author yumingxing
 * @version 1.0
 * @date 2021/2/2 15:19
 **/
@Service
public class EquipmentCheckService implements IEquipmentCheckService {

    @Autowired
    private MesEquipmentCheckRepository equipmentCheckRepository;

    /**
     * 新增
     *
     * @param equipmentCheck
     * @return
     */
    @Override
    public MesEquipmentCheck insert(MesEquipmentCheck equipmentCheck) {
        ConvertBean.serviceModelInitialize(equipmentCheck, AuthUtil.getSessionUser().getUserName());
        equipmentCheck.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(equipmentCheck.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getEquNo(), "equNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getWorkCellCode(), "workCellCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getProjectContent(), "projectContent", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getCheckMethod(), "checkMethod", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getCheckType(), "checkType", ddlPackBean);
        MesEquipmentCheck mesEquipmentCheck = equipmentCheckRepository.getByProperty(ddlPackBean);
        if (!ObjectUtils.isEmpty(mesEquipmentCheck)) {
            throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode(), "数据已存在");
        }

        return equipmentCheckRepository.insert(equipmentCheck);
    }

    /**
     * 修改
     *
     * @param equipmentCheck
     * @return
     */
    @Override
    public MesEquipmentCheck update(MesEquipmentCheck equipmentCheck) {
        MesEquipmentCheck equipmentCheck2 = equipmentCheckRepository.getById(equipmentCheck.getId());
        if (ObjectUtils.isEmpty(equipmentCheck2)) {
            throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode(), "数据不存在");
        }

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getEquNo(), "equNo", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getWorkCellCode(), "workCellCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getWorkCenterCode(), "workCenterCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getProjectContent(), "projectContent", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getCheckMethod(), "checkMethod", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getCheckType(), "checkType", ddlPackBean);
        MesEquipmentCheck mesEquipmentCheck = equipmentCheckRepository.getByProperty(ddlPackBean);
        if (!ObjectUtils.isEmpty(mesEquipmentCheck) && mesEquipmentCheck.getId().compareTo(equipmentCheck.getId()) != 0) {
            throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode(), "数据已存在");
        }

        BeanUtils.copyProperties(equipmentCheck, equipmentCheck2);
        ConvertBean.serviceModelUpdate(equipmentCheck2, AuthUtil.getSessionUser().getUserName());
        return equipmentCheckRepository.save(equipmentCheck2);
    }

    /**
     * 查询
     *
     * @param equipmentCheck
     * @param pager
     * @return
     */
    @Override
    public ListPager<MesEquipmentCheck> queryByPager(MesEquipmentCheck equipmentCheck, Pager pager) {
        DdlPackBean packBean = MesHqlPack.getMesAllData(AuthUtil.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getWorkCellCode(), "workCellCode", packBean);
        DdlPreparedPack.getNumEqualPack(equipmentCheck.getIsValid(), "isValid", packBean);
        DdlPreparedPack.getStringEqualPack(equipmentCheck.getWorkCenterCode(), "workCenterCode", packBean);
        pager = PagerHelper.getPager(pager, equipmentCheckRepository.findByHqlWhereCount(packBean));
        return new ListPager(equipmentCheckRepository.findByHqlWherePage(packBean, pager), pager);
    }

    /**
     * 根据
     *
     * @param id
     * @param status
     * @return
     */
    @Override
    public int updateIsValidById(long id, Integer status) {
        //1、判断单据是否存在
        MesEquipmentCheck equipmentCheck = equipmentCheckRepository.getById(id);
        if (ObjectUtils.isEmpty(equipmentCheck)) {
            throwImppException(CommonEnumUtil.SOFT_TYPE.MES.getCode(), ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode(), "自动配置不存在");
        }

        return equipmentCheckRepository.updateByProperties(new String[]{"id"}, new Object[]{id},
                new String[]{MesConstWords.IS_VALID, "modifyUser", "modifyDatetime"},
                new Object[]{status, AuthUtil.getSessionUser().getUserName(), TimeTool.getNowTime(true)});
    }

    /**
     * 异常
     *
     * @param softTypeCode          softTypeCode
     * @param errorDetailString     错误消息字符串
     * @param imppExceptionEnumCode 异常代码
     * @param args                  参数
     */
    private void throwImppException(String softTypeCode, String errorDetailString, String imppExceptionEnumCode, Object... args) {
        throw ImppExceptionBuilder.newInstance()
                .setSystemID(softTypeCode)
                .setErrorCode(imppExceptionEnumCode)
                .setErrorDetail(errorDetailString, args)
                .build();
    }
}
