package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesEquipmentParam;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesEquipmentParamRepository;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesWorkCenterExtRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.mes.apiservice.util.MesCommonUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCell;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkCellRepository;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 工位设备参数导入，支持编辑
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service
public class EquipmentParamExcelService implements IExcelImportExtService {

    @Autowired
    private MesEquipmentParamRepository equipmentParamRepository;

    @Autowired
    private MesWorkCenterExtRepository workCenterExtRepository;

    @Autowired
    private MesWorkCellRepository workCellRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 工位设备参数导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesEquipmentParam(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesEquipmentParam(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 工位设备参数导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesEquipmentParam(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesEquipmentParam集合
        List<MesEquipmentParam> equipmentParamList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的工作中心代码+工位代码+设备编号+参数编号唯一校验，key：workCenterCode&workCellCode&equNo&paramNo，value：row
        Map<String, String> existMap = new HashMap<>();

        // 查询产线表
        Map<String, Object> workCenterCodeEntityMap;
        try {
            workCenterCodeEntityMap = MesCommonUtil.getCodeEntityMap(workCenterExtRepository, MesExtConstWords.WORK_CENTER_CODE, organizeCode, "产线表");
        } catch (Exception e) {
            workCenterCodeEntityMap = null;
        }

        // 查询工位表
        List<MesWorkCell> workCellList = workCellRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesWorkCell>> workCellCodeEntityMap = CollectionUtils.isEmpty(workCellList) ? null :
                workCellList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getWorkCenterCode() + "&" + o.getWorkCellCode()));

        //查询工位设备参数表
        List<MesEquipmentParam> equipmentParams = equipmentParamRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesEquipmentParam>> equipmentParamMap = CollectionUtils.isEmpty(equipmentParams) ? null :
                equipmentParams.stream().filter(o -> null != o)
                        .collect(Collectors.groupingBy(o -> o.getWorkCenterCode() + "&" + o.getWorkCellCode() + "&" + o.getEquNo() + "&" + o.getParamNo()));


        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //生产线代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String workCenterCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                //工位代码
                String workCellCode = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    workCellCode = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                String workCellName = null;
                //设备编号
                String equNo = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    equNo = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //参数编号
                String paramNo = null;
                try {
                    row.getCell(3).setCellType(CellType.STRING);
                    paramNo = row.getCell(3, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //参数描述
                String paramDesc = null;
                try {
                    row.getCell(4).setCellType(CellType.STRING);
                    paramDesc = row.getCell(4, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //设备名称
                String equName = null;
                try {
                    row.getCell(5).setCellType(CellType.STRING);
                    equName = row.getCell(5, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //参数值
                String paramValue = null;
                try {
                    row.getCell(6).setCellType(CellType.STRING);
                    paramValue = row.getCell(6, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //校验生产线代码
                if (CollectionUtils.isEmpty(workCenterCodeEntityMap) || !workCenterCodeEntityMap.containsKey(workCenterCode)) {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据无效;";
                }

                //校验工位代码
                if (StringUtils.isEmpty(workCellCode)) {
                    errorNum++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                } else {
                    if (StringUtils.isEmpty(errorInfo)) {
                        if (CollectionUtils.isEmpty(workCellCodeEntityMap) || !workCellCodeEntityMap.containsKey(workCenterCode + "&" + workCellCode)) {
                            errorNum++;
                            cellNum += "A+B;";
                            errorInfo += "第A+B列数据无效;";
                        } else {
                            workCellName = workCellCodeEntityMap.get(workCenterCode + "&" + workCellCode).get(0).getWorkCellName();
                        }
                    }
                }

                //校验设备编号
                if (StringUtils.isEmpty(equNo)) {
                    errorNum ++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                }

                //校验参数编号
                if (StringUtils.isEmpty(paramNo)) {
                    errorNum ++;
                    cellNum += "D;";
                    errorInfo += "第D列数据必填;";
                }

                boolean isExist = false;
                //校验工作中心代码+工位代码+设备编号+参数编号不重复
                String key = workCenterCode + "&" + workCellCode + "&" + equNo + "&" + paramNo;
                if (!existMap.containsKey(key)) {
                    existMap.put(key, String.valueOf(rowNum));
                    //校验 工作中心代码+工位代码+设备编号+参数编号 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(equipmentParamMap) && equipmentParamMap.containsKey(key)) {
                        isExist = true;
                    }
                } else {
                    errorNum ++;
                    cellNum += "A+B+C+D;";
                    errorInfo += "第A+B+C+D列数据跟第" + existMap.get(key) + "行的第A+B+C+D列数据重复;";
                }

                //校验参数描述
                if (StringUtils.isEmpty(paramDesc)) {
                    errorNum ++;
                    cellNum += "E;";
                    errorInfo += "第E列数据必填;";
                } else {
                    if (paramDesc.length() > 255) {
                        errorNum++;
                        cellNum += "E;";
                        errorInfo += "第E列数据长度不能超过255个字符;";
                    }
                }

                //校验设备名称
                if (StringUtils.isEmpty(equName)) {
                    errorNum++;
                    cellNum += "F;";
                    errorInfo += "第F列数据必填;";
                }

                //校验参数值
                if (StringUtils.isEmpty(paramValue)) {
                    errorNum++;
                    cellNum += "G;";
                    errorInfo += "第G列数据必填;";
                }

                //没有错误的时候，封装MesEquipmentParam
                if (errorNum == 0) {
                    MesEquipmentParam equipmentParam;
                    if (isExist) {
                        equipmentParam = equipmentParamMap.get(key).get(0);
                        ConvertBean.serviceModelUpdate(equipmentParam, userName);
                    } else {
                        equipmentParam = new MesEquipmentParam();
                        ConvertBean.serviceModelInitialize(equipmentParam, userName);
                    }
                    equipmentParam.setWorkCenterCode(workCenterCode);
                    equipmentParam.setWorkCellCode(workCellCode);
                    equipmentParam.setWorkCellName(workCellName);
                    equipmentParam.setEquNo(equNo);
                    equipmentParam.setParamNo(paramNo);
                    equipmentParam.setParamDesc(paramDesc);
                    equipmentParam.setEquName(equName);
                    equipmentParam.setParamValue(paramValue);
                    equipmentParam.setOrganizeCode(organizeCode);

                    equipmentParamList.add(equipmentParam);

                    successRowNum++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : equipmentParamList);
        return excelImportResultModel;
    }

    /**
     * 工位设备参数扩展导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelMesEquipmentParam(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesEquipmentParam> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesEquipmentParam equipmentParam : excelList) {
                    if(StringUtils.isEmpty(equipmentParam.getId())){
                        equipmentParamRepository.insert(equipmentParam);
                    }else{
                        equipmentParamRepository.save(equipmentParam);
                    }
                }
            }
        }
    }
}
