package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel.bu3;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesProcessQManage;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesProcessQManageRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.mes.apiservice.util.MesCommonUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesProcess;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProcessRepository;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : Q管理项目导入
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service("buQManageExtExcelService")
public class BuQManageExtExcelService implements IExcelImportExtService {

    @Autowired
    private MesProcessQManageRepository mesProcessQManageRDao;

    @Autowired
    private MesProcessRepository mesProcessRDao;

    @Autowired
    private MesPartRepository mesPartRDao;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 工作单元参数配置导入
     *
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesProcessQManage(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesProcessQManage(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 工作单元参数配置导入-读取表格
     *
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesProcessQManage(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesProcessQManage集合
        List<MesProcessQManage> processQManageExtList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的 partNo+工序编号+管理项目+管理内容+检查内容 唯一校验
        Map<String, String> qManageByOnlyMap = new HashMap<>();

        //根据零件号、工序代码、管理项目、管理内容、检查内容 查询替代料中是否存在数据
        List<MesProcessQManage> processQManageList = mesProcessQManageRDao.findByProperty(
                new String[]{MesConstWords.ORGANIZE_CODE, MesConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesProcessQManage>> processQManageMap = CollectionUtils.isEmpty(processQManageList) ? null :
                processQManageList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getProcessCode() + "&" + o.getPartNo() + "&" + o.getManageProjects()+ "&" + o.getScrewType()+ "&" + o.getTighteningTorque() + "&" + o.getCheckType()));

        // 查询物料表
        Map<String, Object> partNoEntityMap;
        try {
            partNoEntityMap = MesCommonUtil.getCodeEntityMap(mesPartRDao, MesExtConstWords.PART_NO, organizeCode, "物料表");
        } catch (Exception e) {
            partNoEntityMap = null;
        }

        // 查询工序表
        Map<String, Object> processCodeEntityMap;
        try {
            processCodeEntityMap = MesCommonUtil.getCodeEntityMap(mesProcessRDao, MesExtConstWords.PROCESS_CODE, organizeCode, "工序表");
        } catch (Exception e) {
            processCodeEntityMap = null;
        }

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //工序编号
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String processCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                String processName = null;
                //零件编号
                String partNo = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    partNo = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //管理项目
                String manageProjects = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    manageProjects = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //管理内容
                String screwType = null;
                try {
                    row.getCell(3).setCellType(CellType.STRING);
                    screwType = row.getCell(3, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //螺批刀编号
                String screwCutterNo = null;
                try {
                    row.getCell(4).setCellType(CellType.STRING);
                    screwCutterNo = row.getCell(4, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //员数
                String qty = null;
                try {
                    row.getCell(5).setCellType(CellType.STRING);
                    qty = row.getCell(5, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //检查内容
                String tighteningTorque = null;
                try {
                    row.getCell(6).setCellType(CellType.STRING);
                    tighteningTorque = row.getCell(6, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //上限值
                String upLimit = null;
                try {
                    row.getCell(7).setCellType(CellType.STRING);
                    upLimit = row.getCell(7, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //下限值
                String downLimit = null;
                try {
                    row.getCell(8).setCellType(CellType.STRING);
                    downLimit = row.getCell(8, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //点检类型
                String checkType = null;
                try {
                    row.getCell(9).setCellType(CellType.STRING);
                    checkType = row.getCell(9, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //校验工序编号
                if (StringUtils.isEmpty(processCode)) {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据必填;";
                } else if (!processCodeEntityMap.containsKey(processCode)) {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据无效;";
                } else {
                    processName = ((MesProcess) processCodeEntityMap.get(processCode)).getProcessName();
                }

                //校验零件编号
                if (!StringUtils.isEmpty(partNo)) {
                    if (!partNoEntityMap.containsKey(partNo)) {
                        errorNum++;
                        cellNum += "B;";
                        errorInfo += "第B列数据无效;";
                    }
                }

                //校验管理项目
                if (StringUtils.isEmpty(manageProjects)) {
                    errorNum++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                }

                //校验管理内容
                if(StringUtils.isEmpty(screwType)){
                    errorNum ++;
                    cellNum += "D;";
                    errorInfo += "第D列数据必填;";
                }

                //校验螺批刀编号
//                if(StringUtils.isEmpty(screwCutterNo)){
//                    errorNum ++;
//                    cellNum += "E;";
//                    errorInfo += "第E列数据必填;";
//                }

                //校验员数
//                if (StringUtils.isEmpty(qty)) {
//                    errorNum++;
//                    cellNum += "F;";
//                    errorInfo += "第F列数据必填;";
//                } else {
//                    try {
//                        Double.parseDouble(qty);
//                    } catch (NumberFormatException e) {
//                        errorNum ++;
//                        cellNum += "F;";
//                        errorInfo += "第F列数据数字类型错误;";
//                    }
//                }

                //校验检查内容
                if (StringUtils.isEmpty(tighteningTorque)) {
                    errorNum++;
                    cellNum += "G;";
                    errorInfo += "第G列数据必填;";
                }

                //校验检查内容
                if (StringUtils.isEmpty(checkType)) {
                    errorNum++;
                    cellNum += "J;";
                    errorInfo += "第J列数据必填;";
                }else if(StringUtils.isEmpty(MesExtEnumUtil.MES_Q_MANAGER_CHECK_TYPE.valueOfDescription(checkType))){
                    errorNum++;
                    cellNum += "J;";
                    errorInfo += "第J列数据无效;";
                }else if (checkType.equals(MesExtEnumUtil.MES_Q_MANAGER_CHECK_TYPE.ORDER_CHECK.getValue()) && StringUtils.isEmpty(partNo)){
                    errorNum++;
                    cellNum += "B;";
                    errorInfo += "当为工单点检时，第B列数据必填;";
                }

                //校验唯一性 不重复
                if (!qManageByOnlyMap.containsKey(processCode + "&" + partNo + "&" + manageProjects+ "&" + screwType+ "&" + tighteningTorque + "&" +checkType)) {
                    qManageByOnlyMap.put(processCode + "&" + partNo + "&" + manageProjects+ "&" + screwType+ "&" + tighteningTorque + "&" + checkType, String.valueOf(rowNum));
                    //根据零件号、工序代码、管理项目、管理内容、检查内容查询替代料中是否存在数据

                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(processQManageMap) && processQManageMap.containsKey(processCode + "&" + partNo + "&" + manageProjects+ "&" + screwType+ "&" + tighteningTorque + "&" + checkType)) {
                        errorNum++;
                        cellNum += "A+B+C+D+G+J;";
                        errorInfo += "第A+B+C+D+G+J列数据已经存在数据库;";
                    }
                } else {
                    errorNum++;
                    cellNum += "A+B+C+D+G;";
                    errorInfo += "第A+B+C+D+G+J列数据跟第" + qManageByOnlyMap.get(processCode + "&" + partNo + "&" + manageProjects+ "&" + screwType+ "&" + tighteningTorque + "&" +checkType) +
                            "行的第A+B+C+D+G+J列数据重复;";
                }

                //校验上限值
                if (!StringUtils.isEmpty(upLimit)) {
                    try {
                        Double.parseDouble(upLimit);
                    } catch (NumberFormatException e) {
                        errorNum++;
                        cellNum += "H;";
                        errorInfo += "第H列数据无效;";
                    }
                    String[] upLimitArr = upLimit.split(MesExtConstWords.DECIMAL_POINT);
                    if (null != upLimitArr && upLimitArr.length == 2 && upLimitArr[1].length() > 2) {
                        errorNum++;
                        cellNum += "H;";
                        errorInfo += "第H列数据无效,最多2位小数;";
                    }

                    if (StringUtils.isEmpty(downLimit)) {
                        errorNum++;
                        cellNum += "H+I;";
                        errorInfo += "第H列跟第I列数据不能只填写其中一个;";
                    }
                }

                //校验上限值
                if (!StringUtils.isEmpty(downLimit)) {
                    try {
                        Double.parseDouble(downLimit);
                    } catch (NumberFormatException e) {
                        errorNum++;
                        cellNum += "I;";
                        errorInfo += "第I列数据无效;";
                    }
                    String[] downLimitArr = downLimit.split(MesExtConstWords.DECIMAL_POINT);
                    if (null != downLimitArr && downLimitArr.length == 2 && downLimitArr[1].length() > 2) {
                        errorNum++;
                        cellNum += "I;";
                        errorInfo += "第I列数据无效,最多2位小数;";
                    }
                    if (StringUtils.isEmpty(upLimit)) {
                        errorNum++;
                        cellNum += "H+I;";
                        errorInfo += "第H列跟第I列数据不能只填写其中一个;";
                    }
                }

                if (StringUtils.isEmpty(errorInfo) && !StringUtils.isEmpty(upLimit) && !StringUtils.isEmpty(downLimit)) {
                    if (MathOperation.compareTo(Double.parseDouble(upLimit), Double.parseDouble(downLimit)) < 0) {
                        errorNum++;
                        cellNum += "H+I;";
                        errorInfo += "第H列数据只能大于等于第I列数据;";
                    }
                }

                //没有错误的时候，封装MesProcessQManage
                if (errorNum == 0) {
                    MesProcessQManage mesProcessQManage = new MesProcessQManage();
                    mesProcessQManage.setPartNo(partNo);
                    mesProcessQManage.setProcessCode(processCode);
                    mesProcessQManage.setProcessName(processName);
                    mesProcessQManage.setManageProjects(manageProjects);
                    mesProcessQManage.setScrewType(screwType);
                    mesProcessQManage.setScrewCutterNo(screwCutterNo);
                    mesProcessQManage.setOrganizeCode(organizeCode);
                    mesProcessQManage.setQty(StringUtils.isEmpty(qty) ? 0 : Double.parseDouble(qty));
                    mesProcessQManage.setTighteningTorque(tighteningTorque);
                    mesProcessQManage.setUpLimit(upLimit);
                    mesProcessQManage.setDownLimit(downLimit);
                    mesProcessQManage.setCheckType(checkType);
                    ConvertBean.serviceModelInitialize(mesProcessQManage, userName);

                    processQManageExtList.add(mesProcessQManage);

                    successRowNum++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultExtModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultExtModel.setExcelList((failRowNum > 0) ? null : processQManageExtList);
        return excelImportResultExtModel;
    }

    /**
     * SOP导入-数据入库
     *
     * @param excelImportResultExtModel
     */
    private void insertExcelMesProcessQManage(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesProcessQManage> processQManageList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(processQManageList)) {
                for (MesProcessQManage processQManage : processQManageList) {
                    mesProcessQManageRDao.insert(processQManage);
                }
            }
        }
    }
}
