package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel.bu3;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.api.base.bu3.ISxWorkLicenseService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesWorkLicense;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesWorkLicenseRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import org.apache.poi.ss.usermodel.DataFormatter;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description :上岗证信息配置导入
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service
public class SxWorkLicenseExcelService implements IExcelImportExtService {

    @Autowired
    private MesWorkLicenseRepository workLicenseRepository;

    @Autowired
    private CommonExtService commonExtService;

    @Autowired
    private ISxWorkLicenseService workLicenseService;

    /**
     * 上岗证配置导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesWorkLicense(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesWorkLicense(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 上岗证配置导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesWorkLicense(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesWorkLicense集合
        List<MesWorkLicense> workLicenseList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        Map<String, String> existMap = new HashMap<>();

        List<MesWorkLicense> workLicenseDbList = workLicenseRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED}, new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesWorkLicense>> workLicenseDbMap = CollectionUtils.isEmpty(workLicenseDbList) ? null :
                workLicenseDbList.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesWorkLicense::getWorkLicenseCode));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //生产线代码
                String workLicenseCode;
                try {
                    workLicenseCode = new DataFormatter().formatCellValue(row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK));
                } catch (Exception e) {
                    continue;
                }

                //姓名
                String name = null;
                try {
                    name = new DataFormatter().formatCellValue(row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK));
                } catch (Exception e) {
                }

                //上岗证有效期
                String validityDate = null;
                try {
                    validityDate = new DataFormatter().formatCellValue(row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK));
                } catch (Exception e) {
                }

                if(!StringUtils.isEmpty(validityDate) && !TimeTool.checkDateFormat(validityDate, MesExtConstWords.DATE_FORMAT)){
                    errorNum ++;
                    cellNum += "C;";
                    errorInfo += "第C列数据格式无效;";
                }

                //备注
                String remark = null;
                try {
                    remark = new DataFormatter().formatCellValue(row.getCell(3, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK));
                } catch (Exception e) {
                }

                Boolean isExist = false;
                if (StringUtils.isEmpty(errorInfo)) {
                    if (!existMap.containsKey(workLicenseCode)) {
                        existMap.put(workLicenseCode, String.valueOf(rowNum));
                        //校验是否已经存在数据库
                        if (!CollectionUtils.isEmpty(workLicenseDbMap) && workLicenseDbMap.containsKey(workLicenseCode)) isExist = true;
                    } else {
                        errorNum ++;
                        cellNum += "A;";
                        errorInfo += "第A列数据跟第" + existMap.get(workLicenseCode) + "行的数据重复;";
                    }
                }

                //没有错误的时候，封装MesWorkLicense
                if (errorNum == 0) {
                    MesWorkLicense workLicense;
                    if (isExist) {
                        workLicense = workLicenseDbMap.get(workLicenseCode).get(0);
                        workLicense.setIsValid(CommonEnumUtil.IS_VAILD.VAILD.getValue());
                        ConvertBean.serviceModelUpdate(workLicense, userName);
                    } else {
                        workLicense = new MesWorkLicense();
                        ConvertBean.serviceModelInitialize(workLicense, userName);
                        workLicense.setWorkLicenseCode(workLicenseCode);
                    }

                    workLicense.setUserName(name);
                    workLicense.setValidityDate(validityDate);
                    workLicense.setRemark(remark);
                    workLicense.setOrganizeCode(organizeCode);
                    workLicense.setSystemSyncStatus(MesExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue());

                    workLicenseList.add(workLicense);

                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : workLicenseList);
        return excelImportResultModel;
    }

    /**
     * 上岗证配置导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelMesWorkLicense(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesWorkLicense> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesWorkLicense workLicense : excelList) {
                    if (StringUtils.isEmpty(workLicense.getId())) {
                        workLicenseRepository.insert(workLicense);
                    } else {
                        workLicenseRepository.save(workLicense);
                    }
                    workLicenseService.insertWorkLicenseTravel(workLicense);
                }
            }
        }
    }
}
